@@ -49,22 +49,6 @@ Config = namedtuple("Config", ["api_id", "api_hash"])
 class Client:
     DIALOGS_AT_ONCE = 100
 
-    CHAT_ACTIONS = {
-        "cancel": types.SendMessageCancelAction,
-        "typing": types.SendMessageTypingAction,
-        "playing": types.SendMessageGamePlayAction,
-        "choose_contact": types.SendMessageChooseContactAction,
-        "upload_photo": types.SendMessageUploadPhotoAction,
-        "record_video": types.SendMessageRecordVideoAction,
-        "upload_video": types.SendMessageUploadVideoAction,
-        "record_audio": types.SendMessageRecordAudioAction,
-        "upload_audio": types.SendMessageUploadAudioAction,
-        "upload_document": types.SendMessageUploadDocumentAction,
-        "find_location": types.SendMessageGeoLocationAction,
-        "record_video_note": types.SendMessageRecordRoundAction,
-        "upload_video_note": types.SendMessageUploadRoundAction,
-    }
-
     def __init__(self, session_name: str, test_mode: bool = False):
         self.session_name = session_name
         self.test_mode = test_mode
@@ -78,6 +62,8 @@ class Client:
         self.peers_by_id = {}
         self.peers_by_username = {}
 
+        self.markdown = Markdown(self.peers_by_id)
+
         self.config = None
         self.session = None
 
@@ -349,31 +335,10 @@ class Client:
             logging.info("Dialogs count: {}".format(len(peers)))
 
         for i in peers:
-            id = i["id"]
-            username = i["username"]
-
-            self.peers_by_id[id] = i
-
-            if username:
-                username = username.lower()
-                self.peers_by_username[username] = i
-
-    def resolve_peer(self, chat_id: int or str):
-        if chat_id in ("self", "me"):
-            input_peer = InputPeerSelf()
-        else:
-            try:
-                peer = (
-                    self.peers_by_username[chat_id.lower()]
-                    if isinstance(chat_id, str)
-                    else self.peers_by_id[chat_id]
-                )
-            except KeyError:
-                raise PeerIdInvalid
-
-            peer_type = peer["type"]
-            peer_id = peer["id"]
-            peer_access_hash = peer["access_hash"]
+            peer_id = i["id"]
+            peer_type = i["type"]
+            peer_username = i["username"]
+            peer_access_hash = i["access_hash"]
 
             if peer_type == "user":
                 input_peer = InputPeerUser(
@@ -390,9 +355,26 @@ class Client:
                     peer_access_hash
                 )
             else:
-                raise PeerIdInvalid
+                continue
 
-        return input_peer
+            self.peers_by_id[peer_id] = input_peer
+
+            if peer_username:
+                peer_username = peer_username.lower()
+                self.peers_by_username[peer_username] = input_peer
+
+    def resolve_peer(self, chat_id: int or str):
+        if chat_id in ("self", "me"):
+            return InputPeerSelf()
+        else:
+            try:
+                return (
+                    self.peers_by_username[chat_id.lower()]
+                    if isinstance(chat_id, str)
+                    else self.peers_by_id[chat_id]
+                )
+            except KeyError:
+                raise PeerIdInvalid
 
     def get_me(self):
         return self.send(
@@ -414,7 +396,7 @@ class Client:
                 silent=disable_notification or None,
                 reply_to_msg_id=reply_to_msg_id,
                 random_id=self.rnd_id(),
-                **Markdown.parse(text)
+                **self.markdown.parse(text)
             )
         )
 
@@ -477,15 +459,12 @@ class Client:
 
     def send_chat_action(self,
                          chat_id: int or str,
-                         action: str,
+                         action: callable,
                          progress: int = 0):
         return self.send(
             functions.messages.SetTyping(
                 peer=self.resolve_peer(chat_id),
-                action=self.CHAT_ACTIONS.get(
-                    action.lower(),
-                    types.SendMessageTypingAction
-                )(progress=progress)
+                action=action(progress=progress)
             )
         )
 
@@ -499,7 +478,7 @@ class Client:
                 peer=self.resolve_peer(chat_id),
                 id=message_id,
                 no_webpage=disable_web_page_preview or None,
-                **Markdown.parse(text)
+                **self.markdown.parse(text)
             )
         )
 
